var _ = require('lodash');
var csv = require('csvtojson');
var path = require('path');
var Promise = require('bluebird');
var request = require('axios');

var readFileAsync = Promise.promisify(require('fs').readFile);

var config = require('./config');

var client = function client(mozaik) {
  mozaik.loadApiConfig(config);

  return {
    list: function list(params) {
      var pathToFile = params.path,
          url = params.url;


      mozaik.logger.info('Fetching table data from ' + (pathToFile || url));

      return pathToFile ? getFSData(params) : getAPIData(params);
    }
  };
};

module.exports = client;

function getFSData(params) {
  var pathToFile = params.path;

  return path.extname(pathToFile) === '.json' ? getJSONFromJSONFile(pathToFile) : getJSONFromCSVFile(pathToFile);
}

function getAPIData(params) {
  var url = params.url,
      _params$headers = params.headers,
      headers = _params$headers === undefined ? {} : _params$headers;


  return request.get(url, { headers: headers }).then(function (res) {
    return _.get(res, 'data');
  });
}

function getJSONFromJSONFile(pathToFile) {
  return readFileAsync(pathToFile).then(function (data) {
    return safeJSONParse(data);
  });
}

function getJSONFromCSVFile(pathToFile) {
  return new Promise(function (resolve, reject) {
    csv().fromFile(pathToFile).on('json', function (data) {
      return resolve(data);
    }).on('done', function (err) {
      return err && reject(err);
    });
  });
}

function safeJSONParse(data) {
  return _.attempt(JSON.parse.bind(null, data));
}