function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

import _ from 'lodash-es';
import React, { Component } from 'react';
import PropTypes from 'prop-types';
import { FaTable as TableIcon } from 'react-icons/fa';
import format from 'string-format';

import { Widget, WidgetHeader, WidgetBody } from '@mozaik/ui';

var styles = {
  table: {
    padding: '0 1rem 0 1rem'
  },
  header: {
    borderBottom: '1px dashed',
    fontWeight: 700,
    marginRight: '50px'
  },
  row: {
    display: 'flex',
    padding: '5px 0 5px 0'
  },
  col: {
    flex: 1
  }
};

var Table = function (_Component) {
  _inherits(Table, _Component);

  function Table() {
    _classCallCheck(this, Table);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  Table.getApiRequest = function getApiRequest(props) {
    return {
      id: 'table.list.' + props.title,
      params: {
        title: props.title,
        path: props.path,
        url: props.url
      }
    };
  };

  Table.prototype.render = function render() {
    var _props = this.props,
        apiData = _props.apiData,
        fields = _props.fields,
        title = _props.title;


    var headers = React.createElement(TableHeaders, { fields: fields });
    var rows = _.map(apiData, function (rowData, rowIndex) {
      return React.createElement(TableRow, { key: rowIndex, fields: fields, data: rowData });
    });

    return React.createElement(
      Widget,
      null,
      React.createElement(WidgetHeader, { title: title, icon: TableIcon }),
      React.createElement(
        WidgetBody,
        null,
        React.createElement(
          'div',
          { style: styles.table },
          React.createElement(
            'ul',
            null,
            headers,
            rows
          )
        )
      )
    );
  };

  return Table;
}(Component);

Table.displayName = 'Table';

Table.propTypes = {
  apiData: PropTypes.array.isRequired,
  fields: PropTypes.array.isRequired,
  path: PropTypes.string,
  title: PropTypes.string.isRequired,
  url: PropTypes.string
};

Table.defaultProps = {
  headers: ['A', 'B', 'C', 'D'],
  fields: ['{A}', '{B}', '{C}', '{D}'],
  title: 'Table'
};

export default Table;

var TableHeaders = function (_React$Component) {
  _inherits(TableHeaders, _React$Component);

  function TableHeaders() {
    _classCallCheck(this, TableHeaders);

    return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  TableHeaders.prototype.render = function render() {
    var fields = this.props.fields;

    var headers = _.map(fields, function (field) {
      var headerText = _.get(_.keys(field), 0);
      return React.createElement(
        'span',
        { style: _.assign({}, styles.header, styles.col) },
        headerText
      );
    });
    return React.createElement(
      'lh',
      { style: styles.row },
      headers
    );
  };

  return TableHeaders;
}(React.Component);

TableHeaders.propTypes = {
  fields: PropTypes.array.isRequired
};

var TableRow = function (_React$Component2) {
  _inherits(TableRow, _React$Component2);

  function TableRow() {
    _classCallCheck(this, TableRow);

    return _possibleConstructorReturn(this, _React$Component2.apply(this, arguments));
  }

  TableRow.prototype.render = function render() {
    var _props2 = this.props,
        fields = _props2.fields,
        data = _props2.data;

    var row = _.map(fields, function (field) {
      var fieldName = _.get(_.keys(field), 0);
      var fieldId = slugify(fieldName);
      var fieldValueTemplate = _.get(field, fieldName);
      var formattedFieldValue = format(fieldValueTemplate, data);
      return React.createElement(
        'span',
        { style: styles.col, key: 'field-' + fieldId },
        formattedFieldValue
      );
    });
    return React.createElement(
      'li',
      { style: styles.row },
      row
    );
  };

  return TableRow;
}(React.Component);

TableRow.propTypes = {
  data: PropTypes.object.isRequired,
  fields: PropTypes.array.isRequired
};

// TODO: don't be lazy, just npm install something?
function slugify(text) {
  return text.replace(/[\W]+/g, ' ').trim().replace(' ', '-');
}