'use strict';

exports.__esModule = true;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _propTypes = require('prop-types');

var _propTypes2 = _interopRequireDefault(_propTypes);

var _fa = require('react-icons/fa');

var _stringFormat = require('string-format');

var _stringFormat2 = _interopRequireDefault(_stringFormat);

var _ui = require('@mozaik/ui');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var styles = {
  table: {
    padding: '0 1rem 0 1rem'
  },
  header: {
    borderBottom: '1px dashed',
    fontWeight: 700,
    marginRight: '50px'
  },
  row: {
    display: 'flex',
    padding: '5px 0 5px 0'
  },
  col: {
    flex: 1
  }
};

var Table = function (_Component) {
  _inherits(Table, _Component);

  function Table() {
    _classCallCheck(this, Table);

    return _possibleConstructorReturn(this, _Component.apply(this, arguments));
  }

  Table.getApiRequest = function getApiRequest(props) {
    return {
      id: 'table.list.' + props.title,
      params: {
        title: props.title,
        path: props.path,
        url: props.url
      }
    };
  };

  Table.prototype.render = function render() {
    var _props = this.props,
        apiData = _props.apiData,
        fields = _props.fields,
        title = _props.title;


    var headers = _react2.default.createElement(TableHeaders, { fields: fields });
    var rows = _lodash2.default.map(apiData, function (rowData, rowIndex) {
      return _react2.default.createElement(TableRow, { key: rowIndex, fields: fields, data: rowData });
    });

    return _react2.default.createElement(
      _ui.Widget,
      null,
      _react2.default.createElement(_ui.WidgetHeader, { title: title, icon: _fa.FaTable }),
      _react2.default.createElement(
        _ui.WidgetBody,
        null,
        _react2.default.createElement(
          'div',
          { style: styles.table },
          _react2.default.createElement(
            'ul',
            null,
            headers,
            rows
          )
        )
      )
    );
  };

  return Table;
}(_react.Component);

Table.displayName = 'Table';

Table.propTypes = {
  apiData: _propTypes2.default.array.isRequired,
  fields: _propTypes2.default.array.isRequired,
  path: _propTypes2.default.string,
  title: _propTypes2.default.string.isRequired,
  url: _propTypes2.default.string
};

Table.defaultProps = {
  headers: ['A', 'B', 'C', 'D'],
  fields: ['{A}', '{B}', '{C}', '{D}'],
  title: 'Table'
};

exports.default = Table;

var TableHeaders = function (_React$Component) {
  _inherits(TableHeaders, _React$Component);

  function TableHeaders() {
    _classCallCheck(this, TableHeaders);

    return _possibleConstructorReturn(this, _React$Component.apply(this, arguments));
  }

  TableHeaders.prototype.render = function render() {
    var fields = this.props.fields;

    var headers = _lodash2.default.map(fields, function (field) {
      var headerText = _lodash2.default.get(_lodash2.default.keys(field), 0);
      return _react2.default.createElement(
        'span',
        { style: _lodash2.default.assign({}, styles.header, styles.col) },
        headerText
      );
    });
    return _react2.default.createElement(
      'lh',
      { style: styles.row },
      headers
    );
  };

  return TableHeaders;
}(_react2.default.Component);

TableHeaders.propTypes = {
  fields: _propTypes2.default.array.isRequired
};

var TableRow = function (_React$Component2) {
  _inherits(TableRow, _React$Component2);

  function TableRow() {
    _classCallCheck(this, TableRow);

    return _possibleConstructorReturn(this, _React$Component2.apply(this, arguments));
  }

  TableRow.prototype.render = function render() {
    var _props2 = this.props,
        fields = _props2.fields,
        data = _props2.data;

    var row = _lodash2.default.map(fields, function (field) {
      var fieldName = _lodash2.default.get(_lodash2.default.keys(field), 0);
      var fieldId = slugify(fieldName);
      var fieldValueTemplate = _lodash2.default.get(field, fieldName);
      var formattedFieldValue = (0, _stringFormat2.default)(fieldValueTemplate, data);
      return _react2.default.createElement(
        'span',
        { style: styles.col, key: 'field-' + fieldId },
        formattedFieldValue
      );
    });
    return _react2.default.createElement(
      'li',
      { style: styles.row },
      row
    );
  };

  return TableRow;
}(_react2.default.Component);

TableRow.propTypes = {
  data: _propTypes2.default.object.isRequired,
  fields: _propTypes2.default.array.isRequired
};

// TODO: don't be lazy, just npm install something?
function slugify(text) {
  return text.replace(/[\W]+/g, ' ').trim().replace(' ', '-');
}